==> emodpy-1.7.0-py3-none-any.whl: tests/test_workflow_campaign.py <==
import os
import pytest
import shutil
import json
from abc import ABC, abstractmethod
from functools import partial

from emod_api.config import default_from_schema_no_validation as dfs
from emod_api.interventions import outbreak as ob
from emod_api.interventions import simple_vaccine as sv
from emod_api.interventions import import_pressure as ip
from emod_api import campaign as camp

from idmtools.entities.experiment import Experiment
from idmtools.core.platform_factory import Platform
from idmtools_test.utils.itest_with_persistence import ITestWithPersistence

from emodpy.emod_task import EMODTask
from emodpy.emod_campaign import EMODCampaign
from emodpy.utils import download_latest_bamboo, download_latest_schema, EradicationBambooBuilds, bamboo_api_login

# import sys
# file_dir = os.path.dirname(__file__)
# sys.path.append(file_dir)
from . import manifest

default_config_file = "default_config.json"  # this is a hard-coded value


# bamboo_api_login() only work in console
# Please run this test from console for the first time or run 'test_download_from_bamboo.py' from console before
# running this test
class TestWorkflowCampaign(ITestWithPersistence, ABC):
    """
        Base test class to test emod_api.campaign and  emod_api.interventions in a workflow
    """
    @classmethod
    @abstractmethod
    def define_test_environment(cls):
        cls.plan = EradicationBambooBuilds.CI_GENERIC
        cls.eradication_path = manifest.eradication_path_win
        cls.schema_path = manifest.schema_path_win
        cls.config_file = os.path.join(manifest.config_folder, "generic_config_for_campaign_workflow.json")
        cls.default_config_file = os.path.join(manifest.config_folder, default_config_file)
        cls.camp_file = os.path.join(manifest.campaign_folder, "generic_campaign.json")
        cls.comps_platform = 'COMPS2'

    @classmethod
    def setUpClass(cls) -> None:
        cls.define_test_environment()
        manifest.delete_existing_file(cls.config_file)
        manifest.delete_existing_file(default_config_file)
        print("write_default_from_schema")
        dfs.write_default_from_schema(cls.schema_path)
        print("move default_config.json")
        shutil.move(default_config_file, cls.default_config_file)
        camp.schema_path = cls.schema_path

    def setUp(self) -> None:
        self.case_name = os.path.basename(__file__) + "--" + self._testMethodName
        print(self.case_name)
        self.get_exe_from_bamboo()
        self.get_schema_from_bamboo()
        self.platform = Platform(self.comps_platform)
        manifest.delete_existing_file(self.camp_file)

    def get_exe_from_bamboo(self):
        if not os.path.isfile(self.eradication_path):
            bamboo_api_login()
            print(
                f"Getting Eradication from bamboo for plan {self.plan}. Please run this script in console if this "
                "is the first time you use bamboo_api_login()."
            )
            eradication_path_bamboo = download_latest_bamboo(
                plan=self.plan,
                scheduled_builds_only=False
            )
            shutil.move(eradication_path_bamboo, self.eradication_path)
        else:
            print(f"{self.eradication_path} already exists, no need to get it from bamboo.")

    def get_schema_from_bamboo(self):
        if not os.path.isfile(self.schema_path):
            bamboo_api_login()
            print(
                f"Getting Schema.json from bamboo for plan {self.plan}. Please run this script in console if this "
                "is the first time you use bamboo_api_login()."
            )
            download_latest_schema(
                plan=self.plan,
                scheduled_builds_only=False,
                out_path=self.schema_path
            )
        else:
            print(f"{self.schema_path} already exists, no need to get it from bamboo.")

    def run_exp(self, task):
        experiment = Experiment.from_task(task, name=self._testMethodName)

        print("Run experiment...")
        experiment.run(platform=self.platform, wait_until_done=True)

        self.assertTrue(experiment.succeeded, msg=f"Experiment {experiment.uid} failed.\n")

        print(f"Experiment {experiment.uid} succeeded.")
        return experiment

    def outbreak_individual_from_file_test(self):
        """
            Testing the campaign.add() to add campagin event from interventions.outbreak and campaign.save() to save
            a campaign file. Making sure it can be consumed by the Eradication with EMODTask.from_files. Make sure
            the following config parameters are set implicitly in config file:
                Campaign_Filename = 'campaign.json'
                Enable_Intervention = 1
        """

        def set_param_fn(config):
            config.parameters.Incubation_Period_Constant = 0
            config.parameters.Infectious_Period_Constant = 1
            config.parameters.Base_Infectivity_Constant = 1
            config.parameters.Base_Mortality = 0
            config.parameters.Simulation_Duration = 10
            return config
        dfs.write_config_from_default_and_params(config_path=self.default_config_file,
                                                 set_fn=set_param_fn,
                                                 config_out_path=self.config_file)
        timestep = 2
        coverage = 0.4
        camp.add(ob.seed_by_coverage(timestep, camp, coverage), name="outbreak_individual", first=True)
        camp.save(self.camp_file)
        self.assertTrue(os.path.isfile(self.camp_file))

        task = EMODTask.from_files(config_path=self.config_file, eradication_path=self.eradication_path,
                                   campaign_path=self.camp_file)
        self.assertTrue(isinstance(task.campaign, EMODCampaign))
        self.assertEqual(len(task.campaign.events), 1)
        self.assertEqual(task.campaign.events[0]["Start_Day"], timestep)
        self.assertEqual(task.campaign.events[0]["Event_Coordinator_Config"]['Demographic_Coverage'], coverage)
        self.assertEqual(task.campaign.events[0]["Event_Coordinator_Config"]['Intervention_Config']['class'],
                         'OutbreakIndividual')

        # these will not be changed until task.pre_creation()
        # self.assertEqual('campaign.json', task.config['Campaign_Filename'])
        # self.assertEqual(1, task.config['Enable_Intervention'])

        experiment = self.run_exp(task)

        for sim in experiment.simulations:
            files = self.platform.get_files(sim, ["config.json", "campaign.json", "stdout.txt"])

            config_file = json.loads(files["config.json"].decode("utf-8"))
            self.assertEqual('campaign.json', config_file['parameters']['Campaign_Filename'])
            self.assertEqual(1, config_file['parameters']['Enable_Interventions'])

            campaign_file = json.loads(files["campaign.json"].decode("utf-8"))
            campaign_file.pop('Campaign_Name')
            with open(self.camp_file, 'r') as camp_file:
                campaign_file_from_disk = json.load(camp_file)
            self.assertEqual(campaign_file, campaign_file_from_disk)

            stdout = files["stdout.txt"].decode("utf-8")
            self.assertIn("'OutbreakIndividual' interventions at node", stdout)

    def ip_and_sv_from_default_test(self):
        """
            Testing the campaign.add() to add campagin event from interventions.import_pressure and
            interventions.simple_vaccine. Making sure it can be consumed by the Eradication with EMODTask.from_default2.
            Make sure the following config parameters are set implicitly in config file:
                Campaign_Filename = 'campaign.json'
                Enable_Intervention = 1
        """

        def set_param_fn(config):
            config.parameters.Incubation_Period_Constant = 0
            config.parameters.Infectious_Period_Constant = 1
            config.parameters.Base_Infectivity_Constant = 1
            config.parameters.Base_Mortality = 0
            config.parameters.Simulation_Duration = 20
            return config

        def build_camp(t, dur, dip, t_sv):
            camp.add(ip.new_intervention(t, dur, dip),
                     name="import_pressure", first=True)
            camp.add(sv.new_intervention(t_sv), name="simple_vaccine", first=False)
            return camp

        sv.schema_path = self.schema_path
        ip.schema_path = self.schema_path
        timestep = 2
        durations = [10, 20]
        daily_import_pressures = [50, 100]
        timestep_sv = 10

        config_path = self.config_file[:-5] + '_2.json'
        task = EMODTask.from_default2(config_path=config_path, eradication_path=self.eradication_path,
                                      campaign_builder=partial(build_camp, timestep, durations, daily_import_pressures,
                                                               timestep_sv),
                                      schema_path=self.schema_path,
                                      param_custom_cb=set_param_fn, ep4_custom_cb=None, demog_builder=None)

        self.assertTrue(isinstance(task.campaign, EMODCampaign))
        self.assertEqual(len(task.campaign.events), 2)
        self.assertEqual(task.campaign.events[0]["Start_Day"], timestep)
        self.assertEqual(task.campaign.events[0]["Event_Coordinator_Config"]['Intervention_Config']['Daily_Import_Pressures'],
                         daily_import_pressures)
        self.assertEqual(task.campaign.events[0]["Event_Coordinator_Config"]['Intervention_Config']['Durations'],
                         durations)
        self.assertEqual(task.campaign.events[0]["Event_Coordinator_Config"]['Intervention_Config']['class'],
                         'ImportPressure')

        self.assertEqual(task.campaign.events[1]["Start_Day"], timestep_sv)
        self.assertEqual(task.campaign.events[1]["Event_Coordinator_Config"]['Intervention_Config']['class'],
                         'SimpleVaccine')

        # these will not be changed until task.pre_creation()
        # self.assertEqual('campaign.json', task.config['Campaign_Filename'])
        # self.assertEqual(1, task.config['Enable_Intervention'])

        experiment = self.run_exp(task)
        config_basename = os.path.basename(config_path)

        for sim in experiment.simulations:
            files = self.platform.get_files(sim, [config_basename, "campaign.json", "stdout.txt"])

            config_file = json.loads(files[config_basename].decode("utf-8"))
            self.assertEqual('campaign.json', config_file['parameters']['Campaign_Filename'])
            self.assertEqual(1, config_file['parameters']['Enable_Interventions'])

            campaign_file = json.loads(files["campaign.json"].decode("utf-8"))
            self.assertEqual(len(campaign_file['Events']), 2)

            stdout = files["stdout.txt"].decode("utf-8")
            self.assertIn("distributed 'ImportPressure' intervention to node", stdout)
            self.assertIn("'SimpleVaccine' interventions at node", stdout)


@pytest.mark.skip('skip tests for Windows Eradication for now')
@pytest.mark.emod
class TestWorkflowCampaignWin(TestWorkflowCampaign):
    """
        Tested with Windows version of Generic Eradication
    """
    @classmethod
    def define_test_environment(cls):
        cls.plan = EradicationBambooBuilds.GENERIC_WIN
        cls.eradication_path = manifest.eradication_path_win
        cls.schema_path = manifest.schema_path_win
        cls.config_file = os.path.join(manifest.config_folder, "generic_config_for_campaign_workflow.json")
        cls.default_config_file = os.path.join(manifest.config_folder, default_config_file)
        cls.camp_file = os.path.join(manifest.campaign_folder, "generic_campaign.json")
        cls.comps_platform = 'COMPS2'

    def test_1_outbreak_individual_from_file_win(self):
        super().outbreak_individual_from_file_test()

    def test_2_ip_and_sv_from_default_win(self):
        super().ip_and_sv_from_default_test()


@pytest.mark.emod
class TestWorkflowCampaignLinux(TestWorkflowCampaign):
    """
        Tested with Linux version of Generic Eradication
    """
    @classmethod
    def define_test_environment(cls):
        cls.plan = EradicationBambooBuilds.GENERIC_LINUX
        cls.eradication_path = manifest.eradication_path_linux
        cls.schema_path = manifest.schema_path_linux
        cls.config_file = os.path.join(manifest.config_folder, "generic_config_for_campaign_workflow_l.json")
        cls.default_config_file = os.path.join(manifest.config_folder, default_config_file)
        cls.camp_file = os.path.join(manifest.campaign_folder, "generic_campaign_l.json")
        cls.comps_platform = 'SLURM'

    def test_1_outbreak_individual_from_file_linux(self):
        super().outbreak_individual_from_file_test()

    def test_2_ip_and_sv_from_default_linux(self):
        super().ip_and_sv_from_default_test()

==> emodpy-1.8.0-py3-none-any.whl: tests/test_workflow_campaign.py <==
import os
import pytest
import shutil
import json
from abc import ABC, abstractmethod
from functools import partial

from emod_api.config import default_from_schema_no_validation as dfs
from emod_api.interventions import outbreak as ob
from emod_api.interventions import simple_vaccine as sv
from emod_api.interventions import import_pressure as ip
from emod_api import campaign as camp

from idmtools.entities.experiment import Experiment
from idmtools.core.platform_factory import Platform
from idmtools_test.utils.itest_with_persistence import ITestWithPersistence

from emodpy.emod_task import EMODTask
from emodpy.emod_campaign import EMODCampaign
from emodpy.utils import download_latest_bamboo, download_latest_schema, EradicationBambooBuilds, bamboo_api_login

# import sys
# file_dir = os.path.dirname(__file__)
# sys.path.append(file_dir)
from . import manifest

default_config_file = "default_config.json"  # this is a hard-coded value


# bamboo_api_login() only work in console
# Please run this test from console for the first time or run 'test_download_from_bamboo.py' from console before
# running this test
class TestWorkflowCampaign(ITestWithPersistence, ABC):
    """
        Base test class to test emod_api.campaign and  emod_api.interventions in a workflow
    """
    @classmethod
    @abstractmethod
    def define_test_environment(cls):
        cls.plan = EradicationBambooBuilds.CI_GENERIC
        cls.eradication_path = manifest.eradication_path_win
        cls.schema_path = manifest.schema_path_win
        cls.config_file = os.path.join(manifest.config_folder, "generic_config_for_campaign_workflow.json")
        cls.default_config_file = os.path.join(manifest.config_folder, default_config_file)
        cls.camp_file = os.path.join(manifest.campaign_folder, "generic_campaign.json")
        cls.comps_platform = 'COMPS2'

    @classmethod
    def setUpClass(cls) -> None:
        cls.define_test_environment()
        manifest.delete_existing_file(cls.config_file)
        manifest.delete_existing_file(default_config_file)
        print("write_default_from_schema")
        dfs.write_default_from_schema(cls.schema_path)
        print("move default_config.json")
        shutil.move(default_config_file, cls.default_config_file)
        camp.schema_path = cls.schema_path

    def setUp(self) -> None:
        self.case_name = os.path.basename(__file__) + "--" + self._testMethodName
        print(self.case_name)
        self.get_exe_from_bamboo()
        self.get_schema_from_bamboo()
        self.platform = Platform(self.comps_platform)
        manifest.delete_existing_file(self.camp_file)

    def get_exe_from_bamboo(self):
        if not os.path.isfile(self.eradication_path):
            bamboo_api_login()
            print(
                f"Getting Eradication from bamboo for plan {self.plan}. Please run this script in console if this "
                "is the first time you use bamboo_api_login()."
            )
            eradication_path_bamboo = download_latest_bamboo(
                plan=self.plan,
                scheduled_builds_only=False
            )
            shutil.move(eradication_path_bamboo, self.eradication_path)
        else:
            print(f"{self.eradication_path} already exists, no need to get it from bamboo.")

    def get_schema_from_bamboo(self):
        if not os.path.isfile(self.schema_path):
            bamboo_api_login()
            print(
                f"Getting Schema.json from bamboo for plan {self.plan}. Please run this script in console if this "
                "is the first time you use bamboo_api_login()."
            )
            download_latest_schema(
                plan=self.plan,
                scheduled_builds_only=False,
                out_path=self.schema_path
            )
        else:
            print(f"{self.schema_path} already exists, no need to get it from bamboo.")

    def run_exp(self, task):
        experiment = Experiment.from_task(task, name=self._testMethodName)

        print("Run experiment...")
        experiment.run(platform=self.platform, wait_until_done=True)

        self.assertTrue(experiment.succeeded, msg=f"Experiment {experiment.uid} failed.\n")

        print(f"Experiment {experiment.uid} succeeded.")
        return experiment

    def outbreak_individual_from_file_test(self):
        """
            Testing the campaign.add() to add campagin event from interventions.outbreak and campaign.save() to save
            a campaign file. Making sure it can be consumed by the Eradication with EMODTask.from_files. Make sure
            the following config parameters are set implicitly in config file:
                Campaign_Filename = 'campaign.json'
                Enable_Intervention = 1
        """

        def set_param_fn(config):
            config.parameters.Incubation_Period_Constant = 0
            config.parameters.Infectious_Period_Constant = 1
            config.parameters.Base_Infectivity_Constant = 1
            config.parameters.Base_Mortality = 0
            config.parameters.Simulation_Duration = 10
            return config
        dfs.write_config_from_default_and_params(config_path=self.default_config_file,
                                                 set_fn=set_param_fn,
                                                 config_out_path=self.config_file)
        timestep = 2
        coverage = 0.4
        camp.add(ob.seed_by_coverage(timestep, camp, coverage), name="outbreak_individual", first=True)
        camp.save(self.camp_file)
        self.assertTrue(os.path.isfile(self.camp_file))

        task = EMODTask.from_files(config_path=self.config_file, eradication_path=self.eradication_path,
                                   campaign_path=self.camp_file)
        self.assertTrue(isinstance(task.campaign, EMODCampaign))
        self.assertEqual(len(task.campaign.events), 1)
        self.assertEqual(task.campaign.events[0]["Start_Day"], timestep)
        self.assertEqual(task.campaign.events[0]["Event_Coordinator_Config"]['Demographic_Coverage'], coverage)
        self.assertEqual(task.campaign.events[0]["Event_Coordinator_Config"]['Intervention_Config']['class'],
                         'OutbreakIndividual')

        # these will not be changed until task.pre_creation()
        # self.assertEqual('campaign.json', task.config['Campaign_Filename'])
        # self.assertEqual(1, task.config['Enable_Intervention'])

        experiment = self.run_exp(task)

        for sim in experiment.simulations:
            files = self.platform.get_files(sim, ["config.json", "campaign.json", "stdout.txt"])

            config_file = json.loads(files["config.json"].decode("utf-8"))
            self.assertEqual('campaign.json', config_file['parameters']['Campaign_Filename'])
            self.assertEqual(1, config_file['parameters']['Enable_Interventions'])

            campaign_file = json.loads(files["campaign.json"].decode("utf-8"))
            campaign_file.pop('Campaign_Name')
            with open(self.camp_file, 'r') as camp_file:
                campaign_file_from_disk = json.load(camp_file)
            self.assertEqual(campaign_file, campaign_file_from_disk)

            stdout = files["stdout.txt"].decode("utf-8")
            self.assertIn("'OutbreakIndividual' interventions at node", stdout)

    def ip_and_sv_from_default_test(self):
        """
            Testing the campaign.add() to add campagin event from interventions.import_pressure and
            interventions.simple_vaccine. Making sure it can be consumed by the Eradication with EMODTask.from_default2.
            Make sure the following config parameters are set implicitly in config file:
                Campaign_Filename = 'campaign.json'
                Enable_Intervention = 1
        """

        def set_param_fn(config):
            config.parameters.Incubation_Period_Constant = 0
            config.parameters.Infectious_Period_Constant = 1
            config.parameters.Base_Infectivity_Constant = 1
            config.parameters.Base_Mortality = 0
            config.parameters.Simulation_Duration = 20
            return config

        def build_camp(t, dur, dip, t_sv):
            camp.add(ip.new_intervention(t, dur, dip),
                     name="import_pressure", first=True)
            camp.add(sv.new_intervention(t_sv), name="vaccine", first=False)
            return camp

        sv.schema_path = self.schema_path
        ip.schema_path = self.schema_path
        timestep = 2
        durations = [10, 20]
        daily_import_pressures = [50, 100]
        timestep_sv = 10

        config_path = self.config_file[:-5] + '_2.json'
        task = EMODTask.from_default2(config_path=config_path, eradication_path=self.eradication_path,
                                      campaign_builder=partial(build_camp, timestep, durations, daily_import_pressures,
                                                               timestep_sv),
                                      schema_path=self.schema_path,
                                      param_custom_cb=set_param_fn, ep4_custom_cb=None, demog_builder=None)

        self.assertTrue(isinstance(task.campaign, EMODCampaign))
        self.assertEqual(len(task.campaign.events), 2)
        self.assertEqual(task.campaign.events[0]["Start_Day"], timestep)
        self.assertEqual(task.campaign.events[0]["Event_Coordinator_Config"]['Intervention_Config']['Daily_Import_Pressures'],
                         daily_import_pressures)
        self.assertEqual(task.campaign.events[0]["Event_Coordinator_Config"]['Intervention_Config']['Durations'],
                         durations)
        self.assertEqual(task.campaign.events[0]["Event_Coordinator_Config"]['Intervention_Config']['class'],
                         'ImportPressure')

        self.assertEqual(task.campaign.events[1]["Start_Day"], timestep_sv)
        self.assertEqual(task.campaign.events[1]["Event_Coordinator_Config"]['Intervention_Config']['class'],
                         'Vaccine')

        # these will not be changed until task.pre_creation()
        # self.assertEqual('campaign.json', task.config['Campaign_Filename'])
        # self.assertEqual(1, task.config['Enable_Intervention'])

        experiment = self.run_exp(task)
        config_basename = os.path.basename(config_path)

        for sim in experiment.simulations:
            files = self.platform.get_files(sim, [config_basename, "campaign.json", "stdout.txt"])

            config_file = json.loads(files[config_basename].decode("utf-8"))
            self.assertEqual('campaign.json', config_file['parameters']['Campaign_Filename'])
            self.assertEqual(1, config_file['parameters']['Enable_Interventions'])

            campaign_file = json.loads(files["campaign.json"].decode("utf-8"))
            self.assertEqual(len(campaign_file['Events']), 2)

            stdout = files["stdout.txt"].decode("utf-8")
            self.assertIn("distributed 'ImportPressure' intervention to node", stdout)
            self.assertIn("'Vaccine' interventions at node", stdout)


@pytest.mark.skip('skip tests for Windows Eradication for now')
@pytest.mark.emod
class TestWorkflowCampaignWin(TestWorkflowCampaign):
    """
        Tested with Windows version of Generic Eradication
    """
    @classmethod
    def define_test_environment(cls):
        cls.plan = EradicationBambooBuilds.GENERIC_WIN
        cls.eradication_path = manifest.eradication_path_win
        cls.schema_path = manifest.schema_path_win
        cls.config_file = os.path.join(manifest.config_folder, "generic_config_for_campaign_workflow.json")
        cls.default_config_file = os.path.join(manifest.config_folder, default_config_file)
        cls.camp_file = os.path.join(manifest.campaign_folder, "generic_campaign.json")
        cls.comps_platform = 'COMPS2'

    def test_1_outbreak_individual_from_file_win(self):
        super().outbreak_individual_from_file_test()

    def test_2_ip_and_sv_from_default_win(self):
        super().ip_and_sv_from_default_test()


@pytest.mark.emod
class TestWorkflowCampaignLinux(TestWorkflowCampaign):
    """
        Tested with Linux version of Generic Eradication
    """
    @classmethod
    def define_test_environment(cls):
        cls.plan = EradicationBambooBuilds.GENERIC_LINUX
        cls.eradication_path = manifest.eradication_path_linux
        cls.schema_path = manifest.schema_path_linux
        cls.config_file = os.path.join(manifest.config_folder, "generic_config_for_campaign_workflow_l.json")
        cls.default_config_file = os.path.join(manifest.config_folder, default_config_file)
        cls.camp_file = os.path.join(manifest.campaign_folder, "generic_campaign_l.json")
        cls.comps_platform = 'SLURM'

    def test_1_outbreak_individual_from_file_linux(self):
        super().outbreak_individual_from_file_test()

    def test_2_ip_and_sv_from_default_linux(self):
        super().ip_and_sv_from_default_test()
